#!/usr/bin/env node
'use strict'; // Require Modules.

require("@babel/polyfill");

var _commander = _interopRequireDefault(require("commander"));

var _chalk = _interopRequireDefault(require("chalk"));

var _didyoumean = _interopRequireDefault(require("didyoumean"));

var _envinfo = _interopRequireDefault(require("envinfo"));

var _updateNotifier = _interopRequireDefault(require("update-notifier"));

var _init = _interopRequireDefault(require("./commands/basic/init"));

var _generate = _interopRequireDefault(require("./commands/basic/generate"));

var _codesplit = _interopRequireDefault(require("./commands/basic/codesplit"));

var _add = _interopRequireDefault(require("./commands/basic/add"));

var _setup = _interopRequireDefault(require("./commands/serve/setup"));

var _docker = _interopRequireDefault(require("./commands/basic/docker"));

var _deploy = _interopRequireDefault(require("./commands/deploy/deploy"));

var _package = _interopRequireDefault(require("../package"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

// Setting edit distance to 60% of the input string's length.
_didyoumean["default"].threshold = 0.6; // Defining action handlers for respective commands.

(0, _updateNotifier["default"])({
  pkg: _package["default"]
}).notify();

var suggestCommands = function suggestCommands(cmd) {
  var availableCommands = _commander["default"].commands.map(function (c) {
    return c._name;
  });

  var suggestion = (0, _didyoumean["default"])(cmd, availableCommands);

  if (suggestion) {
    console.log("  " + _chalk["default"].red("Did you mean ".concat(_chalk["default"].yellow(suggestion), "?")));
  }
}; // Defining all the available commands.


_commander["default"].version(_package["default"].version).usage('<command> [options]');

_commander["default"].command('init <appname>').description('Scaffolds a MEVN stack project in the current path').action(_init["default"]);

_commander["default"].command('codesplit').description('Lazy load components as required').action(_codesplit["default"]);

_commander["default"].command('generate').description('Generates client side component files and server side CRUD boilerplate template').action(_generate["default"]);

_commander["default"].command('add [deps...]').option('-d, --dev', 'install dev-dependencies').description('Install dependencies on the go').action(_add["default"]);

_commander["default"].command('serve').description('Serves client/server locally').action(_setup["default"]);

_commander["default"].command('dockerize').description('Serves the webapp as mult-container Docker applications').action(_docker["default"]);

_commander["default"].command('deploy').description('Deploys the webapp to a cloud solution of choice').action(_deploy["default"]);

_commander["default"].command('info').description('Prints debugging information about the local environment').action(function () {
  console.log(_chalk["default"].bold('\nEnvironment Info:'));

  _envinfo["default"].run({
    System: ['OS', 'CPU'],
    Binaries: ['Node', 'Yarn', 'npm'],
    Browsers: ['Chrome', 'Edge', 'Firefox', 'Safari'],
    npmGlobalPackages: ['mevn-cli']
  }).then(console.log);
});

_commander["default"].arguments('<command>').action(function (cmd) {
  _commander["default"].outputHelp();

  console.log("  " + _chalk["default"].red("\n  Unknown command ".concat(_chalk["default"].yellow(cmd), ".")));
  console.log();
  suggestCommands(cmd);
});

_commander["default"].parse(process.argv); // Shows up help if no arguments were provided.


if (!_commander["default"].args.length) {
  _commander["default"].help();
}