'use strict';

var _chalk = _interopRequireDefault(require("chalk"));

var _fs = _interopRequireDefault(require("fs"));

var _inquirer = _interopRequireDefault(require("inquirer"));

var _nodeBanner = _interopRequireDefault(require("node-banner"));

var _path = _interopRequireDefault(require("path"));

var _projectConfig = _interopRequireDefault(require("../../utils/projectConfig"));

var _messages = require("../../utils/messages");

var _exec = _interopRequireDefault(require("../../utils/exec"));

var _validate = require("../../utils/validate");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

/**
 * Converts a given string into lower camel case
 *
 * @param {String} str - Text to be converted
 * @returns {String}
 */
var toLowerCamelCase = function toLowerCamelCase(str) {
  return str.charAt(0).toUpperCase() + str.substr(1).toLowerCase();
};
/**
 * Generates a Vue SFC
 *
 * @returns {Promise<void>}
 */


var generateComponent = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var progressMsg, _yield$inquirer$promp, componentName, projectConfig, template, isConfigured, _yield$inquirer$promp2, componentType, componentTemplate, componentPath, routesConfigPath, routesConfig, postImportIndex, routesArrayEndsWithIndex, routeConfigToAppend;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return (0, _nodeBanner["default"])('MEVN CLI', 'Light speed setup for MEVN stack based apps.');

          case 2:
            (0, _messages.checkIfConfigFileExists)(); // Message to show up alongwith the spinner

            progressMsg = 'Getting things ready';
            _context.next = 6;
            return _inquirer["default"].prompt([{
              type: 'input',
              name: 'componentName',
              message: 'Please provide a name for the new component',
              validate: _validate.validateInput
            }]);

          case 6:
            _yield$inquirer$promp = _context.sent;
            componentName = _yield$inquirer$promp.componentName;
            // Fetch information specific to the project
            projectConfig = (0, _projectConfig["default"])();
            template = projectConfig.template, isConfigured = projectConfig.isConfigured;
            _context.next = 12;
            return _inquirer["default"].prompt([{
              type: 'list',
              name: 'componentType',
              message: 'Please choose the component type',
              choices: ['UI Component', 'Page Component (generates route)']
            }]);

          case 12:
            _yield$inquirer$promp2 = _context.sent;
            componentType = _yield$inquirer$promp2.componentType;

            // Convert component name to lower-camel-case
            if (!(template === 'Nuxt.js' && componentType !== 'UI Component')) {
              componentName = toLowerCamelCase(componentName);
            } // SFC template


            componentTemplate = ['<template>', '  <center>', '    <br />', "    <h1>This is the ".concat(componentName, " component</h1>"), '  </center>', '</template>', '', '<script>', 'export default {', '  data() {', '    return {}', '  }', '}', '</script>', '', '<style scoped></style>', '']; // Get to know whether the route config is to be touched

            componentPath = '';

            if (componentType === 'UI Component') {
              componentPath = template === 'Nuxt.js' ? 'client/components' : 'client/src/components';
            } else {
              componentPath = template === 'Nuxt.js' ? 'client/pages' : 'client/src/views';
            } // Duplicate component


            if (!_fs["default"].existsSync(_path["default"].join(componentPath, "".concat(componentName, ".vue")))) {
              _context.next = 22;
              break;
            }

            console.log();
            console.log(_chalk["default"].cyan.bold(" Info: ".concat(componentName, ".vue already exists")));
            return _context.abrupt("return");

          case 22:
            _fs["default"].writeFileSync(_path["default"].join(componentPath, "".concat(componentName, ".vue")), componentTemplate.join('\n'));

            console.log(); // Execute linter

            if (isConfigured.client) {
              _context.next = 30;
              break;
            }

            _context.next = 27;
            return (0, _exec["default"])('npm install', progressMsg, 'Successfully installed the dependencies', {
              cwd: 'client'
            });

          case 27:
            progressMsg = 'Cleaning up'; // Update project specific dotfile

            projectConfig.isConfigured.client = true;

            _fs["default"].writeFileSync('.mevnrc', JSON.stringify(projectConfig, null, 2));

          case 30:
            if (!(template === 'Nuxt.js' || componentType === 'UI Component')) {
              _context.next = 34;
              break;
            }

            _context.next = 33;
            return (0, _exec["default"])('npm run lint -- --fix', progressMsg, "Successfully created ".concat(componentName, ".vue file in ").concat(componentPath), {
              cwd: 'client'
            });

          case 33:
            return _context.abrupt("return", _context.sent);

          case 34:
            routesConfigPath = _path["default"].join('client', 'src', 'router.js');
            routesConfig = _fs["default"].readFileSync(routesConfigPath, 'utf8').toString().split('\n');
            postImportIndex = routesConfig.indexOf(routesConfig.find(function (item) {
              return item === '';
            })); // Add an import statement at the respective place

            routesConfig[postImportIndex] = "import ".concat(componentName, " from \"./views/").concat(componentName, ".vue\";"); // Include a new line to compensate the previous addition

            routesConfig.splice(postImportIndex + 1, 0, ''); // Fetch the index corresponding to route-config array closing bracket

            routesArrayEndsWithIndex = routesConfig.indexOf(routesConfig.find(function (item) {
              return item.trim() === ']';
            })); // Append a comma (},) to the previous component route-config delimiter

            routesConfig[routesArrayEndsWithIndex - 1] = '\t},'; // Route config for generated component

            routeConfigToAppend = ['\t{', "\t  path: \"/".concat(componentName.toLowerCase(), "\","), "\t  name: \"".concat(componentName.toLowerCase(), "\","), "\t  component: ".concat(componentName), '\t}']; // Append the route config for newly created component

            routeConfigToAppend.forEach(function (config, index) {
              return routesConfig.splice(routesArrayEndsWithIndex + index, 0, config);
            }); // Write back the updated config

            _fs["default"].writeFileSync(routesConfigPath, routesConfig.join('\n')); // Execute linter


            _context.next = 46;
            return (0, _exec["default"])('npm run lint -- --fix', progressMsg, "Successfully created ".concat(componentName, ".vue file in ").concat(componentPath), {
              cwd: 'client'
            });

          case 46:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function generateComponent() {
    return _ref.apply(this, arguments);
  };
}();

module.exports = generateComponent;