"use strict";

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

/**
 * Copy files
 * @param {any} source - source directory path
 * @param {any} target - path to the destination directory
 * @returns {Void}
 */
var copyFileSync = function copyFileSync(source, target) {
  var targetFile = target; // if target is a directory a new file with the same name will be created

  if (_fs["default"].existsSync(target)) {
    if (_fs["default"].lstatSync(target).isDirectory()) {
      targetFile = _path["default"].join(target, _path["default"].basename(source));
    }
  }

  _fs["default"].writeFileSync(targetFile, _fs["default"].readFileSync(source));
};
/**
 * Copy directory content recursively
 * @param {any} source - source directory path
 * @param {any} target - path to the destination directory
 * @returns {Void}
 */


var copyDirSync = function copyDirSync(source, target) {
  // check if folder needs to be created or integrated
  var targetFolder = _path["default"].join(target, _path["default"].basename(source));

  if (!_fs["default"].existsSync(targetFolder)) {
    _fs["default"].mkdirSync(targetFolder);
  } // copy


  if (_fs["default"].lstatSync(source).isDirectory()) {
    _fs["default"].readdirSync(source).forEach(function (file) {
      var curSource = _path["default"].join(source, file);

      if (_fs["default"].lstatSync(curSource).isDirectory()) {
        copyDirSync(curSource, targetFolder);
      } else {
        copyFileSync(curSource, targetFolder);
      }
    });
  }
};

module.exports = copyDirSync;