'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.hasStrayArgs = exports.directoryExistsInPath = exports.invalidProjectName = exports.showInstallationInfo = exports.dependencyNotInstalled = exports.checkIfTemplateIsNuxt = exports.checkIfConfigFileExists = void 0;

var _chalk = _interopRequireDefault(require("chalk"));

var _fs = _interopRequireDefault(require("fs"));

var _projectConfig = _interopRequireDefault(require("./projectConfig"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

/**
 * Warns appropriately if the config file doesn't exist
 *
 * @returns {Void}
 */
var checkIfConfigFileExists = function checkIfConfigFileExists() {
  if (!_fs["default"].existsSync('./.mevnrc')) {
    console.log(_chalk["default"].cyan.bold("\n\n Make sure that you're within a valid MEVN project\n      \n".concat(_chalk["default"].red.bold(' Error:'), " No .mevnrc file found\n    ")));
    process.exit(1);
  }
};
/**
 * Warns appropriately if the template chosen was Nuxt.js
 * for the respective cases
 *
 * @returns {Promise<void>}
 */


exports.checkIfConfigFileExists = checkIfConfigFileExists;

var checkIfTemplateIsNuxt = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var _appData, template;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _appData = (0, _projectConfig["default"])(), template = _appData.template;

            if (template === 'Nuxt.js') {
              console.log();
              console.log(_chalk["default"].red.bold(" You're having the Nuxt.js boilerplate template"));
              process.exit(0);
            }

          case 2:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function checkIfTemplateIsNuxt() {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Warns appropriately if the respective dependency wasn't installed
 *
 * @param {String} dependency - The dependency to be installed
 * @returns {Void}
 */


exports.checkIfTemplateIsNuxt = checkIfTemplateIsNuxt;

var dependencyNotInstalled = function dependencyNotInstalled(dependency) {
  console.log(_chalk["default"].yellow.bold(" Warning:- ".concat(_chalk["default"].cyan.bold("".concat(dependency, " is required to be installed")), "\n    ")));
  process.exit(0);
};
/**
 * Shows installation information
 *
 * @param {String} depCandidate - The repective package to be installed
 * @param {String} url - Official downloads page url
 * @returns {any}
 */


exports.dependencyNotInstalled = dependencyNotInstalled;

var showInstallationInfo = function showInstallationInfo(depCandidate, url) {
  var msg = " You need to download ".concat(depCandidate, " from the official downloads page: ").concat(url);
  console.log(_chalk["default"].cyan.bold(msg));
  process.exit(0);
};
/**
 * Warns appropriately if the project name is invalid
 *
 * @param {String} projectName - Name of the project
 * @returns {Void}
 */


exports.showInstallationInfo = showInstallationInfo;

var invalidProjectName = function invalidProjectName(projectName) {
  console.log(_chalk["default"].red.bold(" Error: Could not create a project called ".concat(_chalk["default"].red("\"".concat(_chalk["default"].cyan.bold(projectName), "\"")), " because of npm naming restrictions:")));
  process.exit(0);
};
/**
 * Warns appropriately if the respective directory exists in path
 *
 * @param {String} projectName - Name of the project
 * @returns {any}
 */


exports.invalidProjectName = invalidProjectName;

var directoryExistsInPath = function directoryExistsInPath(projectName) {
  console.log(_chalk["default"].red.bold("\n Error: Directory ".concat(_chalk["default"].cyan.bold(projectName), " already exists in path!")));
  process.exit(0);
};
/**
 * Warns the user appropriately if multiple arguments were provided for the project name
 *
 * @returns {Void}
 */


exports.directoryExistsInPath = directoryExistsInPath;

var hasStrayArgs = function hasStrayArgs() {
  console.log(_chalk["default"].red.bold('\n Error: Kindly provide only one argument as the directory name!!'));
  process.exit(0);
};

exports.hasStrayArgs = hasStrayArgs;