'use strict';

var _execa = _interopRequireDefault(require("execa"));

var _inquirer = _interopRequireDefault(require("inquirer"));

var _messages = require("./messages");

var _exec = _interopRequireDefault(require("./exec"));

var _constants = require("./constants");

var _spinner = _interopRequireDefault(require("./spinner"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

// Initialize the spinner.
var spinner = new _spinner["default"]();
/**
 * Helper method to validate installation.
 *
 * @param {String} dependency
 * @returns {Promise<boolean>}
 */

var dependencyIsInstalled = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(dependency) {
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.prev = 0;
            _context.next = 3;
            return _execa["default"].shell(dependency);

          case 3:
            return _context.abrupt("return", true);

          case 6:
            _context.prev = 6;
            _context.t0 = _context["catch"](0);
            return _context.abrupt("return", false);

          case 9:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[0, 6]]);
  }));

  return function dependencyIsInstalled(_x) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Validates user input
 *
 * @param {String} componentName
 * @returns {Boolean}
 */


var validateInput = function validateInput(componentName) {
  if (!componentName) {
    console.log("Can't be empty!");
    return false;
  } else {
    return true;
  }
};
/**
 * Executes respective shell command
 *
 * @param {String} cmd
 * @returns {Promise<any>}
 */

/**
 * Triggers Git installation specific to the platform
 *
 * @returns{Promise<void>}
 */


var installGit = /*#__PURE__*/function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
    var url, packageMgr;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            url = 'https://git-scm.com/download/win';

            if (!_constants.isWin) {
              _context2.next = 5;
              break;
            }

            (0, _messages.showInstallationInfo)('git', url);
            _context2.next = 8;
            break;

          case 5:
            packageMgr = _constants.isLinux ? 'apt' : 'brew';
            _context2.next = 8;
            return (0, _exec["default"])("".concat(packageMgr, " install git"));

          case 8:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function installGit() {
    return _ref2.apply(this, arguments);
  };
}();
/**
 * Triggers Docker installation specific to the platform
 *
 * @returns{Promise<void>}
 */


var installDocker = /*#__PURE__*/function () {
  var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
    var urlMap;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            urlMap = {
              win32: 'https://hub.docker.com/editions/community/docker-ce-desktop-windows',
              darwin: 'https://docs.docker.com/docker-for-mac/install/'
            };

            if (!_constants.isLinux) {
              _context3.next = 6;
              break;
            }

            _context3.next = 4;
            return (0, _exec["default"])('apt install docker.io');

          case 4:
            _context3.next = 7;
            break;

          case 6:
            (0, _messages.showInstallationInfo)('docker', urlMap[process.platform]);

          case 7:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));

  return function installDocker() {
    return _ref3.apply(this, arguments);
  };
}();
/**
 * Installs respective package from the npm registry
 *
 * @returns{Promise<void>}
 */


var installNpmPackage = /*#__PURE__*/function () {
  var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(dependency) {
    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return (0, _exec["default"])("npm install -g ".concat(dependency));

          case 2:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));

  return function installNpmPackage(_x2) {
    return _ref4.apply(this, arguments);
  };
}();
/**
 * Validates installation
 *
 * @param {String} dependency
 * @returns {Promise<boolean>}
 */


var validateInstallation = /*#__PURE__*/function () {
  var _ref5 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(dependency) {
    var status, sep, _yield$inquirer$promp, depToInstall;

    return regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return dependencyIsInstalled(dependency);

          case 2:
            status = _context5.sent;

            if (dependency.includes(' ')) {
              sep = dependency.includes('-') ? '-' : '';
              dependency = dependency.split(sep)[0];
            }

            if (status) {
              _context5.next = 27;
              break;
            }

            _context5.next = 7;
            return _inquirer["default"].prompt([{
              type: 'confirm',
              name: 'depToInstall',
              message: "Sorry, ".concat(dependency, " is not installed on your system, Do you want to install it?")
            }]);

          case 7:
            _yield$inquirer$promp = _context5.sent;
            depToInstall = _yield$inquirer$promp.depToInstall;

            if (!depToInstall) {
              _context5.next = 26;
              break;
            }

            spinner.text = "Installing ".concat(dependency);
            spinner.start();

            if (!(dependency === 'git')) {
              _context5.next = 17;
              break;
            }

            _context5.next = 15;
            return installGit();

          case 15:
            _context5.next = 24;
            break;

          case 17:
            if (!(dependency === 'docker')) {
              _context5.next = 22;
              break;
            }

            _context5.next = 20;
            return installDocker();

          case 20:
            _context5.next = 24;
            break;

          case 22:
            _context5.next = 24;
            return installNpmPackage(dependency);

          case 24:
            _context5.next = 27;
            break;

          case 26:
            (0, _messages.dependencyNotInstalled)(dependency);

          case 27:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5);
  }));

  return function validateInstallation(_x3) {
    return _ref5.apply(this, arguments);
  };
}();

module.exports = {
  validateInstallation: validateInstallation,
  validateInput: validateInput
};